/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 3.0.12
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.sensory.speech.snsr;

/**
 * @file
 * Primary SDK runtime class.
 * @addtogroup java Java API
 * TrulyHandsfree Java SDK.
 *
 * This library uses a [dataflow][] approach to evaluate speech
 * recognition tasks.
 * It relies on [inversion of control][ioc]:
 * User-defined handlers are invoked to report results and control task flow.
 * Task models define flow pipelines and behavior.
 * The API contains two primary data types: @ref java-session
 * and @ref java-stream.
 * Sessions encapsulate the entire state of a task instance, and all
 * session input and output data are are vectored through streams.
 *
 * [dataflow]: https://en.wikipedia.org/wiki/Flow-based_programming
 * [ioc]: https://en.wikipedia.org/wiki/Inversion_of_control
 *
 * Session instance behavior is customized via settings, of which there
 * are three cariants:
 * - **Configuration** settings are serializable read-write values.
 *   These are used for parameter values and model objects.
 *   Task models contain reasonable default values - modify these only to
 *   tweak results.
 * - **Runtime** settings are instance-specific and not serializable. These
 *   include handlers, iterators, input streams and values
 *   such as the name of the phrase being enrolled.
 * - **Result** settings are read-only values with very limited scope, such
 *   as the recognition results.
 *
 * Stream instances are created on various underlying data streams, such as
 * files, memory segments, and live audio sources. Only the
 * constructors are type-specific, all other functions are independent of the
 * underlying stream type.
 *
 * Task evaluation typically follows this recipe:
 * - Create a new session instance.
 * - Load a task model into the instance.
 * - Set the input source stream.
 * - Register one or more event handlers.
 * - Enter the main loop. The library will process the input streams and
 *   invoke event handlers at appropriate times. The main loop continues
 *   until a terminating condition is reached, such as an event returning
 *   an error code.
 * - Release the sesion instance.
 *
 * @{
 * @addtogroup java-session Session
 * Primary SDK API.
 * @{
 */
/**
 * Primary SDK runtime class.
 * A SnsrSession handle contains the entire state for a task.
 *
 * Instance method calls are not re-entrant. If these are called
 * from different threads, all such calls must be protected by
 * application-level synchronization.
 *
 * Methods that are not explicitly used to query the state return
 * the instance itself, to support method chaining. For example:
 * @code
 * SnsrSession s = new SnsrSession()
 *                    .load("task-data.snsr")
 *                    .require(Snsr.TASK_TYPE, Snsr.PHRASESPOT);
 * @endcode
 */
public class SnsrSession {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected SnsrSession(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(SnsrSession obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        SnsrJNI.delete_SnsrSession(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  /**
   * Event listener interface for the SnsrSession class.
   * This interface contains a single method, onEvent() that is
   * invoked by the run() and forEach() methods.
   */
  public interface Listener {
    /**
     * SnsrStream event handler method.
     * This method is invoked by a SnsrSession instance to post events to
     * the caller. The @p key parameter matches the @p key used to
     * register the event (setHandler()) or start the iterator (forEach()).
     *
     * Event callbacks happen in the same thread that the forEach() or
     * run() method was called in. Processing is paused until this method
     * returns, best practice is therefore to have this method return
     * as quickly as possible.
     *
     * Return SnsrRC#OK to continue processing, or any other
     * SnsrRC value to stop.
     *
     * Return codes other than SnsrRC#OK, SnsrRC#STREAM_END, SnsrRC#STOP,
     * SnsrRC#SKIP, SnsrRC#REPEAT, and SnsrRC#TIMED_OUT will
     * cause run() or forEach() to throw an exception.
     *
     * See @ref task for a list of task-specific event and iterator keys
     *
     * @param s the SnsrSession instance that is invoking the method.
     * @param key the identifier for this event type.
     * @return A completion code, use SnsrRC#OK for success.
     * @see forEach(), setHandler(), run().
     */
    public SnsrRC onEvent(SnsrSession s, String key);
  }

  /**
   * Sets the detailed session error message.
   * Sets the detailed error message returned by errorDetail().
   * Use this to propgate a human-readable error message generated
   * by a callback function to the session handle.
   * @param format String.format() style format string.
   * @param args arguments specified by the @p format string.
   */
  public void describeError(String format, Object... args) {
    String msg = String.format(format, args);
    SnsrJNI.snsrDescribeError(swigCPtr, msg);
  }

  /**
   * Releases the native library handle.
   * This method releases the underlying native handles immediately.
   * Use this to free up resources without having to depend on garbage
   * collection.
   *
   * @note No instance methods must be invoked after calling this method.
   *
   * Recommended use:
   * @code
   * SnsrSession s = new SnsrSession();
   * s.release();
   * s = null;
   * @endcode
   */
  public void release() {
    delete();
  }

  
/**
 * Default constructor.
 * Creates a new session instance that does not contain any tasks.
 */
  public SnsrSession() {
    this(SnsrJNI.new_SnsrSession(), true);
  }

  
/**
 * Duplicates a SnsrSession instance.
 * Creates a new SnsrSession instance that is a clone of the source instance.
 * This function is roughly equivalent to:
 * @code
 * SnsrStream buf = SnsrStream.fromBuffer(1024, 1048576);
 * a.save(SnsrDataFormat.CONFIG, buf);
 * SnsrSession b = new SnsrSession().load(buf);
 * buf.delete();
 * buf = null;
 * @endcode
 * The new instance contains all of the configuration from the source,
 * but none of the runtime settings. Immutable configuration settings
 * (such as acoustic models) are shared between the source and the new
 * instance. Using this method to create two instances of a task will
 * therefore use less memory than loading the same task data into two
 * SnsrSession instances.
 *
 * @return A duplicate SnsrSession instance.
 */
  public SnsrSession dup() {
    long cPtr = SnsrJNI.SnsrSession_dup(swigCPtr, this);
    return (cPtr == 0) ? null : new SnsrSession(cPtr, true);
  }

  
/**
 * Retrieves a detailed session error message.
 * This human-readable error message should be used
 * for display or logging only. The content of the message is system-specific.
 * Do not parse this to determine program flow.
 *
 * @return A human-readable error message.
 */
  public String errorDetail() {
    return SnsrJNI.SnsrSession_errorDetail(swigCPtr, this);
  }

  
/**
 * Retrieves the double value associated with a setting.
 *
 * See @ref task for a list of keys, and the contexts in which they
 * are available.
 *
 * @param key setting key, a constant such as Snsr#RES_BEGIN_MS.
 * @return The value for @p key.
 * @see getInt(), setDouble().
 */
  public double getDouble(String key) {
    return SnsrJNI.SnsrSession_getDouble(swigCPtr, this, key);
  }

  
/**
 * Retrieves the integer value associated with a setting.
 *
 * See @ref task for a list of keys, and the contexts in which they
 * are available.
 *
 * @param key setting key, a constant such as Snsr#SPOT_DELAY.
 * @return The value for @p key.
 * @see getDouble(), setInt().
 */
  public int getInt(String key) {
    return SnsrJNI.SnsrSession_getInt(swigCPtr, this, key);
  }

  
/**
 * Retrieves a SnsrStream instance associated with a setting.
 * Returns the SnsrStream value set for @p key in the SnsrSession instance.
 *
 * See @ref task for a list of keys and the contexts
 * in which they are available.
 *
 * @note This function retrieves runtime settings (such as recognition audio or
 * enrolled phrase spot modules) only. It cannot be used to get the stream
 * handles set with setStream().
 *
 * @param key setting key, a constant such as Snsr#MODEL_STREAM.
 * @return A #SnsrStream instance for @p key.
 */
  public SnsrStream getStream(String key) {
    long cPtr = SnsrJNI.SnsrSession_getStream(swigCPtr, this, key);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
/**
 * Retrieves the String value associated with a setting.
 * Returns the String value for @p key in the #SnsrStream instance.
 *
 * See @ref task for a list of keys and the contexts
 * in which they are available.
 *
 * @param  key setting key, a constant such as Snsr#LICENSE_WARNING.
 * @return The String value of @p setting key.
 */
  public String getString(String key) {
    return SnsrJNI.SnsrSession_getString(swigCPtr, this, key);
  }

  
/**
 * Loads a task model or configuration settings into a session.
 * The stream data must be in the format produced by save().
 * @param inputStream readable SnsrStream instance that
 *            contains a task model.
 * @return The SnsrSession instance.
 */
  public SnsrSession load(SnsrStream inputStream) throws java.io.IOException {
    SnsrJNI.SnsrSession_load__SWIG_0(swigCPtr, this, SnsrStream.getCPtr(inputStream), inputStream);
    return this;
  }

  
/**
 * Validates task requirements.
 * Checks that the value of the @p key matches the required value @p value.
 *
 * If @p key is Snsr#TASK_VERSION, this function uses [semantic version][1]
 * precedence to determine whether the task version meets the @p value
 * version requirement.
 *
 * @p value is a semantic version string, optionally prefixed with a comparison
 * operation:
 *  <ul>
 *    <li> @c =, @c <, @c <=, @c >, @c >=: range comparisons.
 *    <li> @c ~: `taskVersion >= value && taskVersion < M` where M is
 *         the next major version number.
 *    <li> @c ^: `taskVersion >= value && taskVersion < M` where M is
 *         the next major version number,
 *         skipping over zeros from the left. For `taskVersion >= 1.0.0`
 *         this is equivalent to the @c ~ operator. `^0.minor.patch`
 *         allows only patch updates, e.g. `^0.1.1` will
 *         accept `0.1.2` but not `0.2.0`.
 *    <li> The default behavior when no operator is specified is @c ^.
 *  </ul>
 *
 * This method is used as a sanity check, to verify that the task model loaded
 * is of the correct type, and that it has a compatible version.
 *
 * @param key the name of a setting to validate, such as Snsr#TASK_TYPE or
 *        Snsr#TASK_VERSION.
 * @param value string to match the @p key setting against, such
 *        as Snsr#PHRASESPOT.
 * @return The SnsrSession instance.
 * @exception IOException
 * [1]: http://semver.orgi/
 */
  public SnsrSession require(String key, String value) {
    SnsrJNI.SnsrSession_require(swigCPtr, this, key, value);
    return this;
  }

  
/**
 * Processes stream data in a session.
 * Enters the main SnsrSession processing loop. This function will process
 * data read from input streams, invoke SnsrSession#Listener registered
 * in the session and (if defined) write to output streams.
 *
 * Processing will continue until:
 *  <ul>
 *   <li> One of the data stream reach SnsrRC#EOF, or
 *   <li> one of the invoked callbacks return an error code.
 *  </ul>
 *
 * Return codes other than SnsrRC#OK, SnsrRC#STREAM_END, SnsrRC#STOP,
 * SnsrRC#SKIP, SnsrRC#REPEAT, and SnsrRC#TIMED_OUT will raise an exception.
 *
 * @return The SnsrSession instance.
 * @throw IOException
 */
  public SnsrSession run() throws java.io.IOException {
    SnsrJNI.SnsrSession_run(swigCPtr, this);
    return this;
  }

  
/**
 * Retrieves the session return code.
 *
 * @return The most recent SnsrRC return code from this session instance.
 */
  public SnsrRC rC() {
    return SnsrRC.swigToEnum(SnsrJNI.SnsrSession_rC(swigCPtr, this));
  }

  
/**
 * Serializes a session to a stream.
 * Saves the configuration or runtime settings of the session instance
 * to an output stream.
 *
 * SnsrDataFormat specifies what is serialized:
 * <ul>
 *   <li> SnsrDataFormat#CONFIG serializes the task and all of its
 *        configuration settings.
 *   <li> SnsrDataFormat#RUNTIME serializes the runtime settings only,
 *        such as phrase spotter enrollments in Snsr#ENROLL tasks.
 * </ul>
 *
 * @param format output format specifier.
 * @param outputStream writable destination stream.
 * @return The SnsrSession instance.
 */
  public SnsrSession save(SnsrDataFormat format, SnsrStream outputStream) {
    SnsrJNI.SnsrSession_save__SWIG_0(swigCPtr, this, format.swigValue(), SnsrStream.getCPtr(outputStream), outputStream);
    return this;
  }

  
/**
 * Changes session configuration.
 * This untility function parses the @p keyValue string and sets
 * session instance configuration options. Use this to change task
 * configuration from user-supplied configuration strings provided to
 * a command-line application.
 *
 * @p keyValue should be of the form "key=value ...", e.g. "delay=30" or
 * "accuracy=0.5 delete-user=hbg".
 *
 * Use setDouble(), setInt() and setString() when the key name is known
 * at compile time, as these functions do not incur a string parsing overhead.
 *
 * @param keyValue setting string, `key=value ...`.
 * @return The SnsrSession instance.
 * @see setDouble(), setInt(), setString().
 */
  public SnsrSession set(String keyValue) {
    SnsrJNI.SnsrSession_set__SWIG_0(swigCPtr, this, keyValue);
    return this;
  }

  
/**
 * Sets a session configuration to a double value.
 * Sets the @p key in this session instance to @p value.
 * @p key must not be `null`.
 *
 * See @ref task for a list of keys and the contexts
 * in which they are available.
 *
 * @param key setting key.
 * @param value `double` value for @p key.
 * @return The SnsrSession instance.
 * @see getDouble(), getInt().
 */
  public SnsrSession setDouble(String key, double value) {
    SnsrJNI.SnsrSession_setDouble(swigCPtr, this, key, value);
    return this;
  }

  
/**
 * Sets a session configuration to an integer value.
 * Sets the @p key in this session instance to @p value.
 * @p key must not be `null`.
 *
 * See @ref task for a list of keys and the contexts
 * in which they are available.
 *
 * @param key setting key.
 * @param value int value for @p key.
 * @return The SnsrSession instance.
 * @see getInt(), setDouble()
 */
  public SnsrSession setInt(String key, int value) {
    SnsrJNI.SnsrSession_setInt(swigCPtr, this, key, value);
    return this;
  }

  
/**
 * Sets input and output streams for a session.
 * Associates @p stream with the source or sink specified by @p key.
 * Stream keys are task-specific and are described in @ref task.
 *
 * Example:
 * @code
 * session.setStream(Snsr.SOURCE_AUDIO_PCM,
 *                   SnsrStream.fromFileName("input.pcm", "r"));
 * @endcode
 *
 * @param key setting key.
 * @param stream a task-specific input or output stream.
 * @return The SnsrSession instance.
 * @see run().
 */
  public SnsrSession setStream(String key, SnsrStream stream) {
    SnsrJNI.SnsrSession_setStream(swigCPtr, this, key, SnsrStream.getCPtr(stream), stream);
    return this;
  }

  
/**
 * Sets a session configuration to a string value.
 * Sets the @p key to @p value. @p key must not be `null`.
 *
 * See @ref task for a list of keys and the contexts
 * in which they are available.

 * @param key setting key.
 * @param value String for @p key.
 * @return The SnsrSession instance.
 */
  public SnsrSession setString(String key, String value) {
    SnsrJNI.SnsrSession_setString(swigCPtr, this, key, value);
    return this;
  }

  
/**
 * Loads a task model or configuration settings into a session.
 * The stream data must be in the format produced by save().
 *
 * @param fileName the path to a file that contains a task model.
 * @return The SnsrSession instance.
 * @throw IOException
 */
  public SnsrSession load(String fileName) throws java.io.IOException {
    SnsrJNI.SnsrSession_load__SWIG_1(swigCPtr, this, fileName);
    return this;
  }

  
/**
 * Serializes a session to file.
 * Saves the configuration or runtime settings of the session instance
 * to the file system.
 *
 * SnsrDataFormat specifies what is serialized:
 * <ul>
 *   <li> SnsrDataFormat#CONFIG serializes the task and all of its
 *        configuration settings.
 *   <li> SnsrDataFormat#RUNTIME serializes the runtime settings only,
 *        such as phrase spotter enrollments in Snsr#ENROLL tasks.
 * </ul>
 *
 * @param format output format specifier.
 * @param fileName output file name.
 * @return The SnsrSession instance.
 */
  public SnsrSession save(SnsrDataFormat format, String fileName) {
    SnsrJNI.SnsrSession_save__SWIG_1(swigCPtr, this, format.swigValue(), fileName);
    return this;
  }

  
/**
 * Sets a session configuration to a double value.
 * Sets the @p key in this session instance to @p value.
 * @p key must not be `null`.
 *
 * See @ref task for a list of keys and the contexts
 * in which they are available.
 *
 * @param key setting key.
 * @param value `double` value for @p key.
 * @return The SnsrSession instance.
 * @see getDouble(), getInt(), setDouble().
 */
  public SnsrSession set(String key, double value) {
    SnsrJNI.SnsrSession_set__SWIG_1(swigCPtr, this, key, value);
    return this;
  }

  
/**
 * Sets a session configuration to an integer value.
 * Sets the @p key in this session instance to @p value.
 * @p key must not be `null`.
 *
 * See @ref task for a list of keys and the contexts
 * in which they are available.
 *
 * @param key setting key.
 * @param value int value for @p key.
 * @return The SnsrSession instance.
 * @see getInt(), setDouble(), setInt().
 */
  public SnsrSession set(String key, int value) {
    SnsrJNI.SnsrSession_set__SWIG_2(swigCPtr, this, key, value);
    return this;
  }

  
/**
 * Sets a session configuration to a string value.
 * Sets the @p key to @p value. @p key must not be `null`.
 *
 * See @ref task for a list of keys and the contexts
 * in which they are available.
 *
 * @param key setting key.
 * @param value String for @p key.
 * @return The SnsrSession instance.
 * @see setString().
 */
  public SnsrSession set(String key, String value) {
    SnsrJNI.SnsrSession_set__SWIG_3(swigCPtr, this, key, value);
    return this;
  }

  
/**
 * Sets input and output streams for a session.
 * Associates @p stream with the source or sink specified by @p key.
 * Stream keys are task-specific and are described in @ref task.
 *
 * Example:
 * @code
 * session.setStream(Snsr.SOURCE_AUDIO_PCM,
 *                   SnsrStream.fromFileName("input.pcm", "r"));
 * @endcode
 *
 * @param key setting key.
 * @param stream a task-specific input or output stream.
 * @return The SnsrSession instance.
 * @see run(), setStream().
 */
  public SnsrSession set(String key, SnsrStream stream) {
    SnsrJNI.SnsrSession_set__SWIG_4(swigCPtr, this, key, SnsrStream.getCPtr(stream), stream);
    return this;
  }

  
/**
 * Invokes a Listener method for each iteration over a list.
 * Invokes @p jcb for each iteration over setting @p key.
 * @p key must reference an iterator setting. These are task-specific -
 * see the @ref task documentation for a list of available iterators.
 *
 * If the listener returns an error (i.e. not SnsrRC#OK) the iteration
 * loop is terminated early, and forEach() returns this result code.
 *
 * @param key iterator key, see task documentation.
 * @param jcb a SnsrSession#Listener invoked for each iteration.
 * @return The SnsrSession instance.
 */
  public SnsrSession forEach(String key, SnsrSession.Listener jcb) {
    SnsrJNI.SnsrSession_forEach(swigCPtr, this, key, jcb);
    return this;
  }

  
/**
 * Registers a session callback handler.
 * The Listener @p jcb will be invoked as specified in the task documentation.
 *
 * See @ref task for details on task-specific callback keys.
 *
 * @param key task-specific callback key.
 * @param jcb a SnsrSession#Listener invoked during run().
 * @return The SnsrSession instance.
 */
  public SnsrSession setHandler(String key, SnsrSession.Listener jcb) {
    SnsrJNI.SnsrSession_setHandler(swigCPtr, this, key, jcb);
    return this;
  }

}

/**
 * @}
 * @}
 */

