/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 3.0.12
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package com.sensory.speech.snsr;

import java.io.IOException;

/**
 * @file
 * Stream abstraction API.
 * @addtogroup java
 * @{
 * @addtogroup java-stream Stream
 * Stream abstraction API.
 * @{
 */
/**
 * Primary stream class.
 *
 * This section describes the stream abstraction through which all
 * SnsrSession inputs and outputs are vectored.
 *
 * The stream API is small and has well-defined semantics. Streams
 * can be created on different underlying data streams (file names, handles,
 * memory segments, live audio sources, and so on).
 * Only the static constructor methods are type-specific,
 * all other functions are independent of the underlying stream type.
 * Operations that take SnsrStream handles as arguments
 * operate on any of the types. The same recognition function, for example,
 * could be used to recognize from file or live input.
 *
 * No operations are done on the underlying data provider until
 * SnsrStream#open() is called, and no operations are done after
 * SnsrStream#close() has been called.
 *
 * @note The current implementation does not do any thread-level locking.
 * Threaded access to stream handles must be protected by explicit
 * synchronization calls.
 */
public class SnsrStream {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected SnsrStream(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(SnsrStream obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        SnsrJNI.delete_SnsrStream(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  /** Default live audio capture sample rate. */
  public static int DEFAULT_SAMPLE_RATE = 16000;

  /**
   * Provider API for user-defined stream types.
   * Use this interface to add new SnsrStream types.
   *
   * Each SnsrStream type exposes a single API function used to create a new
   * instance of this type. The constructor function sets up the required
   * data structures and copies arguments into these structures for use by
   * the Provider#onOpen() method.
   *
   * The constructor must **not** open the underlying data stream.
   * Provider#onOpen() will do this instead. Delaying the open operation
   * allos for function compositions, such as a SnsrStream that is a
   * concatenation of other streams.
   *
   * Use SnsrStream#fromProvider() to create a #SnsrStream instance
   * from a SnsrStream#Provider instance.
   *
   * @see SnsrStream#fromProvider(),
   *      @ref SnsrStreamAudioDeviceAndroid.java,
   * @if JAVA
   *      @ref SnsrStreamAudioDeviceGeneric.java,
   * @endif
   *      SnsrStream#fromAudioDevice(), SnsrStream#fromAudioFile(),
   *      SnsrStream#fromAudioStream(), SnsrStream#fromFileName(),
   *      SnsrStream#fromBuffer(), SnsrStream#fromMemory(),
   *      SnsrStream#fromOpenStream(), SnsrStream#fromStreams(),
   *      SnsrStream#fromString().
   */
  public interface Provider {
    /** Operation successful. */
    public static final long OK = 0;
    /** End-of-stream reached. */
    public static final long EOF = -1;
    /** The read or write operation was interrupted. */
    public static final long INTERRUPTED = -2;
    /** Argument validation failed. */
    public static final long INVALID_ARG = -3;
    /**
     * This operation is not implemented.
     * E.g. a write operation on a read-only stream.
     */
    public static final long NOT_IMPLEMENTED = -4;
    /** Generic error. */
    public static final long ERROR = -5;
    /** Attempt to use a stream that has not been opened. */
    public static final long NOT_OPEN = -6;


    /**
     * Opens the underlying data stream.
     * - Called in response to a SnsrStream#open() API call.
     * - Will only be called on closed SnsrStream handles.
     * - Must open the underlying stream in binary mode, unless documented
     *   otherwise.
     * - Must return Provider#OK if the operation succeeded, and an
     *   error code (Provider#NOT_OPEN) if the resource
     *   could not be opened.
     */
    public long onOpen() throws IOException;

    /**
     * Closes the underlying data stream.
     * - Called in response to a SnsrStream#close() API call.
     * - Will only be called on open SnsrStream handles.
     * - Buffered output must be flushed.
     * - Buffered input must be discarded.
     * - Must return Provider#OK if the operation succeeded,
     *   and an error code if the underlying resource could not be closed.
     */
    public long onClose() throws IOException;

    /**
     * Releases private SnsrStream data.
     * - Called just before the SnsrStream handle is de-allocated.
     * - The handle will have been closed when this function is called -
     *   there is no need to check.
     * - Must release resources allocated by the type's constructor.
     */
    public void onRelease();

    /**
     * Reads data and makes it available as a SnsrStream.
     * - Called in response to a SnsrStream#read() API call.
     * - The handle will be open when this function is called.
     * - Must read @c buffer.length bytes into @p buffer.
     * - The read must block (i.e. not return) until all the requested data
     *   have been read, or an error is encountered.
     * - Must return the number of actual bytes read, unless an error
     *   condition was encountered.
     *     * Returning fewer than @c buffer.length bytes indicates that
     *       the end of the stream was reached.
     * - Must return Provider#INTERRUPTED if the read was interrupted.
     */
    public long onRead(byte[] buffer) throws IOException;

    /**
     * Writes data to the underlying data stream.
     * - Called in response to a SnsrStream#write() API call.
     * - The handle will be open when this function is called.
     * - Must write @c buffer.length bytes from @p buffer.
     * - The write must block (i.e. not return) until all of the
     *   requested data have been written to the underlying stream,
     *   or an error is encountered.
     * - Must return the number of bytes actually written, unless an
     *   error occurred.
     * - Return Provider#INTERRUPTED if the write was interrupted.
     */
    public long onWrite(byte[] buffer) throws IOException;
  }

  public interface Raise {
    public static final long OK = 0;
    public static final long EOF = -1;
    public static final long INTERRUPTED = -2;
    public static final long INVALID_ARG = -3;
    public static final long NOT_IMPLEMENTED = -4;
    public static final long ERROR = -5;
    public static final long NOT_OPEN = -6;
    public long onOpen();
  }

  /**
   * Prints a formatted string to the stream.
   * Formats arguments according to Sting.format() rules, then writes
   * this to the stream.
   * Opens the output stream if it is not already open.
   * If an error occurs, returns @c 0 and sets the
   * stream error state. Use SnsrStream.rC() to check the stream state.
   *
   * @param format String.format type format specifier.
   * @param args as specified by the format specifier.
   * @return The number of bytes written to the stream.
   * @see SnsrStream.rC()
   */
  public long print(String format, Object... args) throws java.io.IOException {
    String msg = String.format(format, args);
    return SnsrJNI.snsrStreamPrint(this.nativeStreamHandle(), msg);
  }

  /**
   * Streams live audio from an audio capture device.
   * Creates a new ::SnsrStream attached to the default audio capture device.
   * The stream will return 16-bit LPCM encoded audio samples in little-endian
   * byte order, sampled at @p sampleRate.
   *
   * @param sampleRate Audio capture sampling rate.
   * @return A new ::SnsrStream handle.
   */
  public static SnsrStream fromAudioDevice(int sampleRate) {
    return SnsrStream.fromProvider(new SnsrStreamAudioDevice(sampleRate),
                                 SnsrStreamMode.READ);
  }

  /**
   * Streams live audio at the default sample rate.
   * Creates a new ::SnsrStream attached to the default audio capture device.
   * The stream will return 16-bit LPCM encoded audio samples in little-endian
   * byte order, sampled at the default rate.
   *
   * Equivalent to:
   * @code
   * SnsrStream a = SnsrStream.fromAudioDevice(SnsrStream.DEFAULT_SAMPLE_RATE);
   * @endcode
   *
   * @return A new ::SnsrStream handle.
   * @see SnsrStream::fromAudioDevice(int sampleRate)
   */
  public static SnsrStream fromAudioDevice() {
    return SnsrStream.fromAudioDevice(DEFAULT_SAMPLE_RATE);
  }

  /**
   * Creates a read-only or write-only stream from a SnsrStream.Provider.
   *
   * The returned stream is either read-only or write-only, as specified
   * by the @p mode parameter.
   *
   * Example:
   * @code
   * public static SnsrStream fromAudioDevice(int sampleRate) {
   *   return SnsrStream.fromProvider(new SnsrStreamAudioDevice(sampleRate),
   *                                SnsrStreamMode.READ);
   * }
   * @endcode
   *
   * @param custom plugin instance.
   * @param mode specifies whether the stream is read-only or write-only.
   * @return A new ::SnsrStream handle.
   */
  public static SnsrStream fromProvider(SnsrStream.Provider custom,
                                      SnsrStreamMode mode) {
    SnsrStream a = new SnsrStream();
    switch (mode) {
    case READ:  a.alloc(custom, 1, 0); break;
    case WRITE: a.alloc(custom, 0, 1); break;
    }
    return a;
  }

  /**
   * Creates a read-write stream from a SnsrStream.Provider.
   *
   * The returned stream supports both reading and writing.
   *
   * @param custom plugin instance.
   * @return A new ::SnsrStream handle.
   */
  public static SnsrStream fromProvider(SnsrStream.Provider custom) {
    SnsrStream a = new SnsrStream();
    a.alloc(custom, 1, 1);
    return a;
  }

  /**
   * Releases the native library handle.
   * This method releases the underlying native handles immediately.
   * Use this to free up resources without having to depend on garbage
   * collection.
   *
   * @note No instance methods must be invoked after calling this method.
   *
   * Recommended use:
   * @code
   * SnsrStream s = SnsrStream.fromAudioDevice();
   * s.release();
   * s = null;
   * @endcode
   */
  public void release() {
    delete();
  }

  public void alloc(SnsrStream.Provider jstrm, int readable, int writable) {
    SnsrJNI.SnsrStream_alloc(swigCPtr, this, jstrm, readable, writable);
  }

  public long nativeStreamHandle() {
    return SnsrJNI.SnsrStream_nativeStreamHandle(swigCPtr, this);
  }

  public static SnsrStream raise(SnsrStream.Raise jevent) {
    long cPtr = SnsrJNI.SnsrStream_raise(jevent);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  public SnsrStream() {
    this(SnsrJNI.new_SnsrStream(), true);
  }

  
  /**
   * Reports stream end status.
   * Returns @c 1 if the stream has reached its end.
   * * Follows Unix semantics:
   *   - The end-of-stream indicator is only valid after a read attempt.
   *   - A read on a ::SnsrStream @c b could very well return @c 0, even if
   *     @c snsrStreamAtEnd(b) returned @c 0 just before attempting the read.
   * * Equivalent to @code (SnsrStream.rC(b) == SnsrRC.EOF) @endcode
   *
   * @return @c 1 if at end-of-stream, 0 if not.
   * @see SnsrStream.rC()
   */
  public int atEnd() {
    return SnsrJNI.SnsrStream_atEnd(swigCPtr, this);
  }

  
  /**
   * Closes an open stream.
   * - Any remaining buffered output is flushed.
   * - Any remaining buffered input is discarded.
   * - Closing a stream that is not open is not an error.
   * - Streams that are still open when the last reference is removed
   *   are closed before the handle is released.
   *
   * @return SnsrRC#OK for success, any other value indicates failure.
   */
  public SnsrRC close() {
    return SnsrRC.swigToEnum(SnsrJNI.SnsrStream_close(swigCPtr, this));
  }

  /**
   * Copies from a stream.
   * Opens streams if they are not already open, 
   * copies @p sizeInBytes bytes from @p src, then returns the
   * number of bytes successfully written.
   * If an error occurred (including an end-of-stream condition)
   * the return value will be less than @p sizeInBytes.
   * 
   * @param src source stream.
   * @param sizeInBytes number of bytes to copy from @p src.
   * @return The number of bytes actually copied.
   * @see SnsrStream.rC()
   */
  public long copy(SnsrStream src, long sizeInBytes) throws java.io.IOException {
    return SnsrJNI.SnsrStream_copy__SWIG_0(swigCPtr, this, SnsrStream.getCPtr(src), src, sizeInBytes);
  }

  
  /**
   * Retrieves a detailed stream error message.
   * - This human-readable error message containing the reason for failure.
   * - Use for display or logging only, as the content of the message is
   *   unspecified and system-specific.
   * - Do not parse this to determine program flow,
   *   use the SnsrRC return code instead.
   *
   * @return A detailed message describing the most recent reason for failure.
   */
  public String errorDetail() {
    return SnsrJNI.SnsrStream_errorDetail(swigCPtr, this);
  }

  
  /**
   * Reads one line from the stream.
   * Reads a line from the stream, delimited by the character @p delim into
   * @p buffer.
   * - Opens the output stream if it is not already open.
   * - Reads up to `buffer.length - 1` bytes from the stream
   *   into @p buffer, stops when @p delim has been read (@p buffer will
   *   include @p delim).
   * - Terminates the string in @p buffer with @c \0 and
   *   returns the number of characters read (not including the terminating
   *    @c \0).
   * - If the delimiter is not found after reading
   *   `buffer.length - 1` bytes, the stream
   *   error code is set to SnsrRC#DELIM_NOT_FOUND.
   * - `buffer.length >= 2` is required.
   * - @c buffer will contain all characters read and will be terminated
   *   with @c \0, even if an error occurs.
   *
   * @param buffer destination buffer.
   * @param delim delimiter character to search for.
   * @return The number of characters read and placed into @p buffer.
   */
  public long getDelim(byte[] buffer, int delim) throws java.io.IOException {
    return SnsrJNI.SnsrStream_getDelim(swigCPtr, this, buffer, delim);
  }

  
  /**
   * Queries stream metadata.
   * Returns stream metadata, selected by @p key.
   * If an error occurs, this fuction returns @c 0 and sets the stream error
   * state. Use SnsrStream.rC() to check stream state.
   */
  public long getMeta(SnsrStreamMeta key) {
    return SnsrJNI.SnsrStream_getMeta(swigCPtr, this, key.swigValue());
  }

  
  /**
   * Opens a stream.
   * - Read and write operations are valid only on open streams.
   * - Newly created streams are not open.
   * - Streams are always opened in binary mode.
   * - Some stream types can be closed and re-opened. This behaviour is
   *   in the static constructor methods.
   * - If re-opening is not supported SnsrStream.open() will return
   *   SnsrRC#CANNOT_REOPEN when an attempt is made to open it a second time.
   *
   * @return SnsrRC#OK for success, any other value indicates failure.
   * @throw IOException
   */
  public SnsrRC open() throws java.io.IOException {
    return SnsrRC.swigToEnum(SnsrJNI.SnsrStream_open(swigCPtr, this));
  }

  
  /**
   * Retrieves the most recent return code for the stream.
   * @return The SnsrRC code for the stream.
   */
  public SnsrRC rC() {
    return SnsrRC.swigToEnum(SnsrJNI.SnsrStream_rC(swigCPtr, this));
  }

  /**
   * Reads from a stream into a buffer at an offset
   * - Opens stream if it is not already open.
   * - Reads @p count bytes into @p buffer at @p offset.
   * - Returns the number of bytes read.
   * - A short read (less than @p count) will occur only
   *    if the end of the stream was reached before @p count bytes were read,
   *    or if an error occurred. The stream return code will be set.
   *    Use SnsrStream.atEnd() or SnsrStream.rC() to distinguish between these
   *    conditions.
   * - Read operations block until all the requested data have been read, or
   *   an error or end-of-stream is encountered.
   * 
   * @param buffer destination byte[] buffer.
   * @param offset from the start of buffer where writing will commence.
   * @param count number of bytes to read from the stream.
   * @return The number of bytes read.
   */
  public long read(byte[] buffer, long offset, long count) throws java.io.IOException {
    return SnsrJNI.SnsrStream_read__SWIG_0(swigCPtr, this, buffer, offset, count);
  }

  /**
   * Reads from a stream, filling a buffer.
   * - Opens stream if it is not already open.
   * - Reads bytes into @p buffer, attempts to fill the entire buffer.
   * - Returns the number of bytes read.
   * - A short read (less than `buffer.length`) will occur only if the end of
   *   the stream was reached before `buffer.length` bytes were read, or if an
   *   error occurred. The stream return code will be set.
   *   Use SnsrStream.atEnd()or SnsrStream.rC() to determine which occurred.
   * - Read operations block until all the requested data have been read, or
   *   an error or end-of-stream is encountered.
   * 
   * @param buffer destination byte[] buffer.
   * @return The number of bytes read.
   */
  public long read(byte[] buffer) throws java.io.IOException {
    return SnsrJNI.SnsrStream_read__SWIG_1(swigCPtr, this, buffer);
  }

  /**
   * Reads one byte from a stream.
   * - Opens stream if it is not already open.
   * - Returns the value of the byte read, or
   *   Integer.MIN_VALUE if an error occurred.
   * - If the end of the stream was reached, or an error occurred, the
   *   stream return code status will be set. Use SnsrStream.atEnd() or
   *    SnsrStream.rC() to distinguish between these conditions.
   * - Read operations block until a byte has been read, or
   *   an error or end-of-stream is encountered.
   * 
   * @return The number of bytes read.
   */
  public int read() throws java.io.IOException {
    return SnsrJNI.SnsrStream_read__SWIG_2(swigCPtr, this);
  }

  /**
   * Reads from a stream and discards the results.
   * Opens stream if it is not already open, 
   * reads @p nitems, each of size @p size from the stream and discards
   * the read data.
   * Returns the number of items (not bytes) read.
   * 
   * @param size the size of an item, in bytes.
   * @param nitems the number of items to read and discard.
   * @return The number of items read.
   * @see SnsrStream.read()
   */
  public long skip(long size, long nitems) throws java.io.IOException {
    return SnsrJNI.SnsrStream_skip__SWIG_0(swigCPtr, this, size, nitems);
  }

  /**
   * Reads from a stream and discards the results.
   * Opens stream if it is not already open, 
   * reads @p nitems bytes from the stream and discards the read data.
   * Returns the number of bytes read.
   * 
   * @param nitems the number of bytes to read and discard.
   * @return The number of bytes read.
   * @see SnsrStream.read()
   */
  public long skip(long nitems) throws java.io.IOException {
    return SnsrJNI.SnsrStream_skip__SWIG_1(swigCPtr, this, nitems);
  }

  /**
   * Writes from a buffer to the stream.
   * - Opens stream if it is not already open.
   * - Writes @p count bytes to the stream  obtaining them from @p buffer
   *   starting at @p offset bytes. Returns the number of bytes written.
   * - Returns less than @p count only if an error occurred. Use
   *   SnsrStream.rC() to obtain the error code.
   * - Write operations block until the requested data have been transferred
   *   to the underlying implementation, but may return well before the
   *   data have reached their final destination (written to disk, played from
   *    a speaker, etc.)
   * 
   * @param buffer source byte[] buffer.
   * @param offset from the start of buffer where reading will start.
   * @param count number of bytes to write.
   * @return The number of bytes written.
   */
  public long write(byte[] buffer, long offset, long count) throws java.io.IOException {
    return SnsrJNI.SnsrStream_write__SWIG_0(swigCPtr, this, buffer, offset, count);
  }

  /**
   * Writes an entire buffer to the stream.
   * - Opens stream if it is not already open.
   * - Writes `buffer.length` bytes to the stream obtaining them from @p buffer
   *   Returns the number of bytes written.
   * - Returns less than `buffer.length` only if an error occurred. Use
   *   SnsrStream.rC() to obtain the error code.
   * - Write operations block until the requested data have been transferred
   *   to the underlying implementation, but may return well before the
   *   data have reached their final destination (written to disk, played from
   *    a speaker, etc.)
   * 
   * @param buffer source byte[] buffer.
   * @return The number of bytes written.
   */
  public long write(byte[] buffer) throws java.io.IOException {
    return SnsrJNI.SnsrStream_write__SWIG_1(swigCPtr, this, buffer);
  }

  /**
   * Writes an single byte to the stream.
   * - Opens stream if it is not already open.
   * - Writes @p oneByte to the stream.
   * - The stream return code will be set if and end-of-stream condition
   *   or other error occurred. Use SnsrStream.rC() to obtain the error code.
   * - Write operations block until the requested data have been transferred
   *   to the underlying implementation, but may return well before the
   *   data have reached their final destination (written to disk, played from
   *    a speaker, etc.)
   * 
   * @param oneByte a single source byte.
   * @return The number of bytes written - either zero or one.
   */
  public long write(int oneByte) throws java.io.IOException {
    return SnsrJNI.SnsrStream_write__SWIG_2(swigCPtr, this, oneByte);
  }

  
 /**
  * Wraps a stream around an Android asset.
  * Creates a new read-only ::SnsrStream on @p filename in the compressed assets
  * section of an Android application.
  * @p filename is relative to `assets/` and should not include this as part
  * of the name.
  * @note Call Snsr#init() before the first call to this function.
  *
  * @note Available on Android SDKs only.
  * @param filename path to the asset, relative to the `assets/` directory.
  * @return A new ::SnsrStream handle.
  * [1]: https://developer.android.com/ndk/reference/group___asset.html
  */
 public static SnsrStream fromAsset(String filename) {
    long cPtr = SnsrJNI.SnsrStream_fromAsset(filename);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
 /**
  * Wraps a stream around a RIFF WAVE file.
  * This function is equivalent to:
  * @code
  * SnsrStream s;
  * s = SnsrStream.fromAudioStream(SnsrStream.fromFileName(filename, mode),
  *                                SnsrStreamAudioFormat.DEFAULT);
  * @endcode
  *
  * @param filename the name of the .wav file to open.
  * @param mode file access mode.
  * @return A new ::SnsrStream handle.
  * @see SnsrStream::fromAudioStream(), SnsrStream::fromFileName().
  */
 public static SnsrStream fromAudioFile(String filename, String mode) {
    long cPtr = SnsrJNI.SnsrStream_fromAudioFile(filename, mode);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
 /**
  * Converts stream audio format.
  * Creates a new ::SnsrStream on an existing stream, @p a. The new
  * stream produces audio in @p format, converting as needed.
  *
  * The audio format header in @p a will be read when the stream is
  * first opened.
  * * If this is a known, but unsupported format, SnsrStream::open() will
  *   report an error.
  * * If the format is not known, the data in @p a will be treated as
  *   a headerless little-endian 16-bit LPCM stream.
  *
  * The initial implementation supports the 16-bit LPCM RIFF Wave format,
  * at 16 kHz sampling rate, for a single channel only.
  *
  * @param a source ::SnsrStream.
  * @param format output target format.
  * @return A new ::SnsrStream handle.
  */
 public static SnsrStream fromAudioStream(SnsrStream a, SnsrStreamAudioFormat format) {
    long cPtr = SnsrJNI.SnsrStream_fromAudioStream(SnsrStream.getCPtr(a), a, format.swigValue());
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
 /**
  * Wraps a stream around a stdio file.
  * Creates a new ::SnsrStream using the @c &lt;stdio.h&gt; file functions.
  *
  * Only a subset of the @c fopen() modes are supported:
  *   - @c r for read-only access.
  *   - @c w for write-only access, truncating the file to zero on open.
  *   - @c a for write-only append access.
  *
  * This stream type supports re-opening. The behaviour follows that of
  * @c fopen() with the given access mode.
  *
  * @param filename the name of a file to open.
  * @param mode     @c r, @c w, or @c a.
  * @return A new ::SnsrStream handle.
  */
 public static SnsrStream fromFileName(String filename, String mode) {
    long cPtr = SnsrJNI.SnsrStream_fromFileName(filename, mode);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
 /**
  * Wraps a stream around a circular buffer.
  * Creates a new ::SnsrStream on a circular buffer with first-in-first-out
  * semantics.
  * The initial buffer will be large enough to hold @p initialSizeInBytes
  * bytes, and will dynamically grow up to @p maxSizeInBytes.
  * If the ::SnsrStream the bufffer cannot be expanded, SnsrStream#write() will
  * set the SnsrRC#EOF condition on the stream.
  * This ::SnsrStream type does not support re-opening.
  * @note SnsrStream#read() on streams of this type has non-blocking behavior,
  *  and will return SnsrRC#EOF if the buffer runs empty.
  *
  * @param initialSizeInBytes the minimum size of the allocated ring buffer.
  * @param maxSizeInBytes the limit the buffer is allowed to expand to.
  * @return A new ::SnsrStream handle.
  */
 public static SnsrStream fromBuffer(long initialSizeInBytes, long maxSizeInBytes) {
    long cPtr = SnsrJNI.SnsrStream_fromBuffer(initialSizeInBytes, maxSizeInBytes);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
 /**
  * Wraps a stream around a memory segment.
  * Creates a new ::SnsrStream on memory segment @p store.
  * This ::SnsrStream type does not support re-opening.
  *
  * @note Changes written through the SnsrStream API
  *   will not be visible in @p store until the SnsrStream
  *   instance has been released.
  * @note Read behavior is undefined if @p store is modified
  *   after the SnsrStream is instantiated.
  *
  * @param store data made available to the stream.
  * @param mode  open for reading or writing, but not both.
  * @return A new ::SnsrStream handle.
  * @see SnsrStream::fromString().
  */
 public static SnsrStream fromMemory(byte[] store, SnsrStreamMode mode) {
    long cPtr = SnsrJNI.SnsrStream_fromMemory(store, mode.swigValue());
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
 /**
  * Wraps a stream around an open stream.
  * Creates a new ::SnsrStream on an existing, open ::SnsrStream.
  * The new stream mode (read or write) matches the @p source stream.
  * Read and write operations are passed through to the @p source stream, but
  * the number of bytes read or written is limited to @p sizeInBytes.
  * In- and output operations will set the stream status to SnsrRC#EOF
  * when this limit is reached.
  *
  * Closing this ::SnsrStream instance will not close @p source.
  *
  * @param source source stream.
  * @param sizeInBytes new stream read or write limit.
  * @return A new ::SnsrStream handle.
  */
 public static SnsrStream fromOpenStream(SnsrStream source, long sizeInBytes) {
    long cPtr = SnsrJNI.SnsrStream_fromOpenStream(SnsrStream.getCPtr(source), source, sizeInBytes);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
 /**
  * Concatenates two streams.
  * Creates a new ::SnsrStream that is a concatenation of @p a and @p b.
  *
  * Streams @p a and @p b will be each be opened exactly once. These should
  * be unopened (SnsrStream#getMeta() for SnsrStreamMeta#OPEN_COUNT must be
  * @c 0) when SnsrStream#fromStreams() is called.
  * This limitation does not apply to streams returned by this function,
  * these can be nested to arbitrary depth to concatenate more than two
  * streams.
  *
  * The new stream will read from (or write to) @p a until end-of-stream
  * is reached, then switch to stream @p b until that is exhausted,
  * at which point SnsrRC#EOF is returned.
  *
  * Streams @p a and @p b must have identical modes. If one stream was opened
  * for reading, and the other for writing, this function will set the
  * error code to SnsrRC#WRONG_MODE.
  *
  * Closing this stream will close the current source stream. The next
  * open call will open the next source stream. When no further source
  * streams remain SnsrStream.open() returns SnsrRC#CANNOT_REOPEN.
  *
  * @param a first source stream.
  * @param b second source stream.
  * @return A new ::SnsrStream handle.
  */
 public static SnsrStream fromStreams(SnsrStream a, SnsrStream b) {
    long cPtr = SnsrJNI.SnsrStream_fromStreams(SnsrStream.getCPtr(a), a, SnsrStream.getCPtr(b), b);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  
 /**
  * Wraps a stream around a read-only string.
  * Creates a new ::SnsrStream from a zero-terminated String.
  *
  * @note Read behavior is undefined if @p store is modified
  *   after the SnsrStream is instantiated.
  *
  * @param store source data made available to the stream.
  * @return A new ::SnsrStream handle.
  * @see SnsrStream::fromMemory().
  */
 public static SnsrStream fromString(String store) {
    long cPtr = SnsrJNI.SnsrStream_fromString(store);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  public void setDetail(String detail) {
    SnsrJNI.SnsrStream_setDetail(swigCPtr, this, detail);
  }

  public void setRC(SnsrRC code) {
    SnsrJNI.SnsrStream_setRC(swigCPtr, this, code.swigValue());
  }

  public SnsrStream concat(SnsrStream src) {
    long cPtr = SnsrJNI.SnsrStream_concat__SWIG_0(swigCPtr, this, SnsrStream.getCPtr(src), src);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  public SnsrStream concat(String filename) {
    long cPtr = SnsrJNI.SnsrStream_concat__SWIG_1(swigCPtr, this, filename);
    return (cPtr == 0) ? null : new SnsrStream(cPtr, true);
  }

  /**
   * Copies all available data from a stream.
   * Opens streams if they are not already open, 
   * copies all available bytes from @p src, then returns the number of bytes
   * successfully copied.
   * 
   * @param src source stream.
   * @return The number of bytes copied.
   * @see SnsrStream.rC()
   */
  public long copy(SnsrStream src) throws java.io.IOException {
    return SnsrJNI.SnsrStream_copy__SWIG_1(swigCPtr, this, SnsrStream.getCPtr(src), src);
  }

}

/**
 * @}
 * @}
 */

